! ***********************************************************************
!
!   Copyright (C) 2010  Bill Paxton
!
!   this file is part of mesa.
!
!   mesa is free software; you can redistribute it and/or modify
!   it under the terms of the gnu general library public license as published
!   by the free software foundation; either version 2 of the license, or
!   (at your option) any later version.
!
!   mesa is distributed in the hope that it will be useful, 
!   but without any warranty; without even the implied warranty of
!   merchantability or fitness for a particular purpose.  see the
!   gnu library general public license for more details.
!
!   you should have received a copy of the gnu library general public license
!   along with this software; if not, write to the free software
!   foundation, inc., 59 temple place, suite 330, boston, ma 02111-1307 usa
!
! ***********************************************************************
 
      module run_star_extras

      use star_lib
      use star_def
      use const_def
      use math_lib
      use gyre_lib
      use utils_lib

      implicit none

      include "test_suite_extras_def.inc"

    real(dp), target, dimension(:) :: actual_Qs(430) !=>NULL()
    real(dp), target, dimension(:) :: actual_neuQs(430) !=>NULL() ! ignore if null  (num_reactions)
    logical, target :: from_weaklib(430)! ! ignore if null
!
!      real(dp) :: temp49
!      real(dp) :: temp50
!      real(dp) :: temp51
!      real(dp) :: temp52
!      real(dp) :: temp53
!      real(dp) :: temp54
!      real(dp) :: temp55
!      real(dp) :: temp56
!      real(dp) :: temp57

! here are the x controls used below

!gyre
!      x_logical_ctrl(37) = .false. ! if true, then run GYRE
!      x_integer_ctrl(1) = 2 ! output GYRE info at this step interval
!      x_logical_ctrl(1) = .false. ! save GYRE info whenever save profile
!      x_integer_ctrl(2) = 2 ! max number of modes to output per call
!      x_logical_ctrl(2) = .false. ! output eigenfunction files
!      x_integer_ctrl(3) = 0 ! mode l (e.g. 0 for p modes, 1 for g modes)
!      x_integer_ctrl(4) = 1 ! order
!      x_ctrl(1) = 0.158d-05 ! freq ~ this (Hz)
!      x_ctrl(2) = 0.33d+03 ! growth < this (days)

!alpha_mlt_routine
         !alpha_H = s% x_ctrl(21)
         !alpha_other = s% x_ctrl(22)
         !H_limit = s% x_ctrl(23)
      
      contains

      include "test_suite_extras.inc"
      
      
      subroutine extras_controls(id, ierr)
         integer, intent(in) :: id
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         s% extras_startup => extras_startup
         s% extras_check_model => extras_check_model
         s% extras_finish_step => extras_finish_step
         s% extras_after_evolve => extras_after_evolve
         s% how_many_extra_history_columns => how_many_extra_history_columns
         s% data_for_extra_history_columns => data_for_extra_history_columns
         s% how_many_extra_profile_columns => how_many_extra_profile_columns
         s% data_for_extra_profile_columns => data_for_extra_profile_columns  
         s% other_alpha_mlt => alpha_mlt_routine       
      end subroutine extras_controls


      subroutine alpha_mlt_routine(id, ierr)
         use chem_def, only: ih1
         integer, intent(in) :: id
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         integer :: k, h1
         real(dp) :: alpha_H, alpha_other, H_limit
         include 'formats'
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         alpha_H = s% x_ctrl(21)
         alpha_other = s% x_ctrl(22)
         H_limit = s% x_ctrl(23)
         h1 = s% net_iso(ih1)
         !write(*,1) 'alpha_H', alpha_H
         !write(*,1) 'alpha_other', alpha_other
         !write(*,1) 'H_limit', H_limit
         !write(*,2) 'h1', h1
         !write(*,2) 's% nz', s% nz
         if (alpha_H <= 0 .or. alpha_other <= 0 .or. h1 <= 0) return
         do k=1,s% nz
            if (s% xa(h1,k) >= H_limit) then
               s% alpha_mlt(k) = alpha_H
            else
               s% alpha_mlt(k) = alpha_other
            end if
            !write(*,2) 'alpha_mlt', k, s% alpha_mlt(k), 
         end do
         !stop
      end subroutine alpha_mlt_routine
      
      
      subroutine extras_startup(id, restart, ierr)
         integer, intent(in) :: id
         logical, intent(in) :: restart
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         call test_suite_startup(s, restart, ierr)         
            
         if (.not. s% x_logical_ctrl(37)) return
         
         ! Initialize GYRE

         call gyre_init('gyre.in')

         ! Set constants

         call gyre_set_constant('G_GRAVITY', standard_cgrav)
         call gyre_set_constant('C_LIGHT', clight)
         call gyre_set_constant('A_RADIATION', crad)

         call gyre_set_constant('M_SUN', Msun)
         call gyre_set_constant('R_SUN', Rsun)
         call gyre_set_constant('L_SUN', Lsun)

         call gyre_set_constant('GYRE_DIR', TRIM(mesa_dir)//'/gyre/gyre')

      end subroutine extras_startup
      
      
      subroutine extras_after_evolve(id, ierr)
         use num_lib, only: find0
         integer, intent(in) :: id
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         real(dp) :: dt, m
         integer :: k, nz
         include 'formats'
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         nz = s% nz
         write(*,*)
         call test_suite_after_evolve(s, ierr)
         if (ierr /= 0) return         
         if (.not. s% x_logical_ctrl(37)) return
         call gyre_final()
      end subroutine extras_after_evolve
      

      ! returns either keep_going, retry, or terminate.
      integer function extras_check_model(id)
         integer, intent(in) :: id
         integer :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         extras_check_model = keep_going         
      end function extras_check_model


      integer function how_many_extra_history_columns(id)
         integer, intent(in) :: id
         integer :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         how_many_extra_history_columns = 90
      end function how_many_extra_history_columns


subroutine data_for_extra_history_columns(id, n, names, vals, ierr)

   use star_def, only: star_info
   use const_def, only: dp
   use net_lib, only: net_work_size
   use rates_lib, only: rates_reaction_id
   integer, intent(in) :: id, n
   character (len=maxlen_history_column_name) :: names(n)
   real(dp) :: vals(n), temp(n), frac(n),baryonic_mass(n)
   real(dp) :: integral_norm, gamma1_integral, P_tot
   real(dp) :: nps_beta, nps_invbeta, eps_beta, eps_invbeta
   integer, intent(out) :: ierr
   integer :: num_weak_reactions
   integer :: i, k, op_err, net_lwork
   logical :: okay
   character (len=maxlen_history_column_name) :: names_weak_reactions(n)
   character (len=maxlen_history_column_name) :: names_weak_ps(n)
   type (star_info), pointer :: s
   ierr = 0
   call star_ptr(id, s, ierr)
   if (ierr /= 0) return

num_weak_reactions = 40


! new variables will be placed above the reactions.
! in low mass inlist, num_weak_reactions =

! 24 beta decay reactions
names_weak_reactions(1) = 'r_h1_he3_wk_he4'
names_weak_reactions(2) = 'r_h1_h1_ec_h2'
names_weak_reactions(3) = 'r_h1_h1_wk_h2'
names_weak_reactions(4) = 'r_h1_wk_neut'

names_weak_reactions(5) = 'r_be7_wk_li7'

names_weak_reactions(6) = 'r_b8_wk_he4_he4'

names_weak_reactions(7) = 'r_n13_wk_c13'

names_weak_reactions(8) = 'r_o14_wk_n14'
names_weak_reactions(9) = 'r_o15_wk_n15'

names_weak_reactions(10) = 'r_f17_wk_o17'
names_weak_reactions(11) = 'r_f18_wk_o18'

names_weak_reactions(12) = 'r_ne18_wk_f18'
names_weak_reactions(13) = 'r_ne19_wk_f19'

names_weak_reactions(14) = 'r_na21_wk_ne21'
names_weak_reactions(15) = 'r_na22_wk_ne22'

names_weak_reactions(16) = 'r_mg22_wk_na22'
names_weak_reactions(17) = 'r_mg23_wk_na23'
names_weak_reactions(18) = 'r_mg24_wk_na24'

names_weak_reactions(19) = 'r_al25_wk_mg25'
names_weak_reactions(20) = 'r_al26_wk_mg26'

names_weak_reactions(21) = 'r_si26_wk_al26'
names_weak_reactions(22) = 'r_si27_wk_al27'

names_weak_reactions(23) = 'r_p30_wk_si30'

names_weak_reactions(24) = 'r_s31_wk_p31'


! 16 inverse beta decay reactions
names_weak_reactions(25) = 'r_neut_wk-minus_h1'

names_weak_reactions(26) = 'r_o17_wk-minus_f17'
names_weak_reactions(27) = 'r_o18_wk-minus_f18'

names_weak_reactions(28) = 'r_f18_wk-minus_ne18'
names_weak_reactions(29) = 'r_f19_wk-minus_ne19'

names_weak_reactions(30) = 'r_ne21_wk-minus_na21'
names_weak_reactions(31) = 'r_ne22_wk-minus_na22'


names_weak_reactions(32) = 'r_na22_wk-minus_mg22'
names_weak_reactions(33) = 'r_na23_wk-minus_mg23'
names_weak_reactions(34) = 'r_na24_wk-minus_mg24'

names_weak_reactions(35) = 'r_mg25_wk-minus_al25'
names_weak_reactions(36) = 'r_mg26_wk-minus_al26'

names_weak_reactions(37) = 'r_al26_wk-minus_si26'
names_weak_reactions(38) = 'r_al27_wk-minus_si27'

names_weak_reactions(39) = 'r_si30_wk-minus_p30'

names_weak_reactions(40) = 'r_p31_wk-minus_s31'



!neutrinos per second

! 24 beta decay reactions
names_weak_ps(1) = 'nps_h1_he3_wk_he4'
names_weak_ps(2) = 'nps_h1_h1_ec_h2'
names_weak_ps(3) = 'nps_h1_h1_wk_h2'
names_weak_ps(4) = 'nps_h1_wk_neut'

names_weak_ps(5) = 'nps_be7_wk_li7'

names_weak_ps(6) = 'nps_b8_wk_he4_he4'

names_weak_ps(7) = 'nps_n13_wk_c13'

names_weak_ps(8) = 'nps_o14_wk_n14'
names_weak_ps(9) = 'nps_o15_wk_n15'

names_weak_ps(10) = 'nps_f17_wk_o17'
names_weak_ps(11) = 'nps_f18_wk_o18'

names_weak_ps(12) = 'nps_ne18_wk_f18'
names_weak_ps(13) = 'nps_ne19_wk_f19'

names_weak_ps(14) = 'nps_na21_wk_ne21'
names_weak_ps(15) = 'nps_na22_wk_ne22'

names_weak_ps(16) = 'nps_mg22_wk_na22'
names_weak_ps(17) = 'nps_mg23_wk_na23'
names_weak_ps(18) = 'nps_mg24_wk_na24'

names_weak_ps(19) = 'nps_al25_wk_mg25'
names_weak_ps(20) = 'nps_al26_wk_mg26'

names_weak_ps(21) = 'nps_si26_wk_al26'
names_weak_ps(22) = 'nps_si27_wk_al27'

names_weak_ps(23) = 'nps_p30_wk_si30'

names_weak_ps(24) = 'nps_s31_wk_p31'


! 16 inverse beta decay reactions
names_weak_ps(25) = 'nps_neut_wk-minus_h1'

names_weak_ps(26) = 'nps_o17_wk-minus_f17'
names_weak_ps(27) = 'nps_o18_wk-minus_f18'

names_weak_ps(28) = 'nps_f18_wk-minus_ne18'
names_weak_ps(29) = 'nps_f19_wk-minus_ne19'

names_weak_ps(30) = 'nps_ne21_wk-minus_na21'
names_weak_ps(31) = 'nps_ne22_wk-minus_na22'


names_weak_ps(32) = 'nps_na22_wk-minus_mg22'
names_weak_ps(33) = 'nps_na23_wk-minus_mg23'
names_weak_ps(34) = 'nps_na24_wk-minus_mg24'

names_weak_ps(35) = 'nps_mg25_wk-minus_al25'
names_weak_ps(36) = 'nps_mg26_wk-minus_al26'

names_weak_ps(37) = 'nps_al26_wk-minus_si26'
names_weak_ps(38) = 'nps_al27_wk-minus_si27'

names_weak_ps(39) = 'nps_si30_wk-minus_p30'

names_weak_ps(40) = 'nps_p31_wk-minus_s31'



!erg per second from each weak reactions

! 24 beta decay reactions
names_weak_ps(1+ num_weak_reactions) = 'eps_h1_he3_wk_he4'
names_weak_ps(2+ num_weak_reactions) = 'eps_h1_h1_ec_h2'
names_weak_ps(3+ num_weak_reactions) = 'eps_h1_h1_wk_h2'
names_weak_ps(4+ num_weak_reactions) = 'eps_h1_wk_neut'

names_weak_ps(5+ num_weak_reactions) = 'eps_be7_wk_li7'

names_weak_ps(6+ num_weak_reactions) = 'eps_b8_wk_he4_he4'

names_weak_ps(7+ num_weak_reactions) = 'eps_n13_wk_c13'

names_weak_ps(8+ num_weak_reactions) = 'eps_o14_wk_n14'
names_weak_ps(9+ num_weak_reactions) = 'eps_o15_wk_n15'

names_weak_ps(10+ num_weak_reactions) = 'eps_f17_wk_o17'
names_weak_ps(11+ num_weak_reactions) = 'eps_f18_wk_o18'

names_weak_ps(12+ num_weak_reactions) = 'eps_ne18_wk_f18'
names_weak_ps(13+ num_weak_reactions) = 'eps_ne19_wk_f19'

names_weak_ps(14+ num_weak_reactions) = 'eps_na21_wk_ne21'
names_weak_ps(15+ num_weak_reactions) = 'eps_na22_wk_ne22'

names_weak_ps(16+ num_weak_reactions) = 'eps_mg22_wk_na22'
names_weak_ps(17+ num_weak_reactions) = 'eps_mg23_wk_na23'
names_weak_ps(18+ num_weak_reactions) = 'eps_mg24_wk_na24'

names_weak_ps(19+ num_weak_reactions) = 'eps_al25_wk_mg25'
names_weak_ps(20+ num_weak_reactions) = 'eps_al26_wk_mg26'

names_weak_ps(21+ num_weak_reactions) = 'eps_si26_wk_al26'
names_weak_ps(22+ num_weak_reactions) = 'eps_si27_wk_al27'

names_weak_ps(23+ num_weak_reactions) = 'eps_p30_wk_si30'

names_weak_ps(24+ num_weak_reactions) = 'eps_s31_wk_p31'


! 16 inverse beta decay reactions
names_weak_ps(25+ num_weak_reactions) = 'eps_neut_wk-minus_h1'

names_weak_ps(26+ num_weak_reactions) = 'eps_o17_wk-minus_f17'
names_weak_ps(27+ num_weak_reactions) = 'eps_o18_wk-minus_f18'

names_weak_ps(28+ num_weak_reactions) = 'eps_f18_wk-minus_ne18'
names_weak_ps(29+ num_weak_reactions) = 'eps_f19_wk-minus_ne19'

names_weak_ps(30+ num_weak_reactions) = 'eps_ne21_wk-minus_na21'
names_weak_ps(31+ num_weak_reactions) = 'eps_ne22_wk-minus_na22'


names_weak_ps(32+ num_weak_reactions) = 'eps_na22_wk-minus_mg22'
names_weak_ps(33+ num_weak_reactions) = 'eps_na23_wk-minus_mg23'
names_weak_ps(34+ num_weak_reactions) = 'eps_na24_wk-minus_mg24'

names_weak_ps(35+ num_weak_reactions) = 'eps_mg25_wk-minus_al25'
names_weak_ps(36+ num_weak_reactions) = 'eps_mg26_wk-minus_al26'

names_weak_ps(37+ num_weak_reactions) = 'eps_al26_wk-minus_si26'
names_weak_ps(38+ num_weak_reactions) = 'eps_al27_wk-minus_si27'

names_weak_ps(39+ num_weak_reactions) = 'eps_si30_wk-minus_p30'

names_weak_ps(40+ num_weak_reactions) = 'eps_p31_wk-minus_s31'




baryonic_mass = (/&
3d0,1d0,1d0,1d0,&
7d0,&
8d0,&
13d0,&
14d0,15d0,&
17d0,18d0,&
18d0,19d0,&
21d0,22d0,&
22d0,23d0,24d0,&
25d0,26d0,&
26d0,27d0,&
30d0,&
31d0,&

1d0,&
17d0,18d0,&
18d0,19d0,&
21d0,22d0,&
22d0,23d0,24d0,&
25d0,26d0,&
26d0,27d0,&
30d0,&
31d0/)


do i=1,80
    names(i) = names_weak_ps(i)
end do


names(81)='log_Lneu_plas' ! units of Lsun
names(82)='log_Lneu_brem' ! units of Lsun
names(83)='log_Lneu_phot' ! units of Lsun
names(84)='log_Lneu_pair' ! units of Lsun
names(85)='log_Lneu_reco' ! units of Lsun
names(86)='gamma1_integral' ! integrated <gamma1-4/3>
names(87) = 'log_Lneu_beta' ! units of Lsun
names(88) = 'log_Lneu_invbeta' ! units of Lsun
names(89) = 'log_nps_beta' ! units of 1/s
names(90) = 'log_nps_invbeta' ! units of 1/s






!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! We begin the calculation here:
! set initial values to 0
integral_norm = 0d0
gamma1_integral = 0d0
do i=1,90
    vals(i) = 0
end do

!!!!!! here, do you want to calculate rates? okay = .true.
! calculare values or not. yes or no.
okay = .true.
! $OMP PARALLEL DO PRIVATE(k,op_err)
net_lwork = net_work_size(s% net_handle, ierr)


do k = 1, s% nz
    !if (.not. okay) cycle
    op_err = 0
    ! set initial values to 0
    do i=1,90
    temp(i) = vals(i)
    end do


    ! now do rates
    ! when this function gets called it sets all the vals(n)
    !to new values associated with the screening mode in current cell k
    call do2_net( &
    s, k, s% species, &
    s% num_reactions, net_lwork, n, s% nz, vals, frac,names_weak_reactions, ierr)
    if (op_err /= 0) okay = .false.





    !neutrinos/second for each reaction
    ! write(*,*) 'test h1 mass fraction', h1_frac
    ! Here vals have been called, so before each loop we can modify vals
    ! multiply the appropriate quantity for each rate by vals(#)
    ! s% xa(g% net_iso(reaction_inputs(2*f,rate_id)),k) example f is factor
    do i =1,num_weak_reactions ! for all weak reactions in mesa_136.net except first 3.
        if (i == 1) then
        vals(1) = exp10(vals(1)) * (frac(2) * frac(1) * s% dm(k) * avo /(baryonic_mass(2) + baryonic_mass(1))) ! neutrinos/s
     !   write (*,*) 'vals(1)', vals(1)
      ! write (*,*) '1vals(149)', vals(149)
        vals(41) = vals(41) * vals(1) /avo ! ergs/s
        vals(41) = temp(41) + vals(41)
        vals(1) = temp(1) + vals(1)

        else if (i ==2) then
        vals(2) = exp10(vals(2)) * (frac(2)**2 * s% dm(k) * avo /2d0 /baryonic_mass(2)) ! 1/s
      !  write (*,*) 'vals(2)', vals(2)
       ! write (*,*) '1vals(150)', vals(150)
        vals(42) = vals(42) * vals(2) /avo ! ergs/s, vals(28) = Qneutrino * conversion_factor_to_mev_qconv(Mev_to_ergs*avo)
        vals(42) = temp(42) + vals(42)
        vals(2) = temp(2) + vals(2)


        else if (i ==3) then
        vals(3) = exp10(vals(3)) * (frac(2)**2 * s% dm(k) * avo /2d0 /baryonic_mass(2)) ! 1/s
        !write (*,*) 'vals(3)', vals(3)
       ! write (*,*) '1vals(151)', vals(151)
        vals(43) = vals(43) * vals(3) /avo ! ergs/s, vals(28) = Qneutrino * conversion_factor_to_mev_qconv(Mev_to_ergs*avo)
        vals(43) = temp(43) + vals(43)
        vals(3) = temp(3) + vals(3)


        else
            vals(i) = exp10(vals(i)) /baryonic_mass(i) * frac(i) * s% dm(k) * avo ! neutrinos/s
    
            vals(i+num_weak_reactions) = vals(i+num_weak_reactions) * vals(i) /avo ! ergs/s
    !        write (*,*) '2vals(i+num_weak_reactions)', vals(i+num_weak_reactions)
            vals(i+num_weak_reactions) = temp(i+num_weak_reactions) + vals(i+num_weak_reactions)
     !       write (*,*) '3vals(i+num_weak_reactions)', vals(i+num_weak_reactions)
            vals(i) = temp(i) + vals(i)

        end if
    end do
    ! calulcating log L and gamma1
    ! set ending variables here:
    vals(81) = temp(81) + s% nonnucneu_plas(k) * s% dm(k)
    vals(82) = temp(82) + s% nonnucneu_brem(k) * s% dm(k)
    vals(83) = temp(83) + s% nonnucneu_phot(k) * s% dm(k)
    vals(84) = temp(84) + s% nonnucneu_pair(k) * s% dm(k)
    vals(85) = temp(85) + s% nonnucneu_reco(k) * s% dm(k)

    ! compute gamma integral in what will remain of the star
    ! temp302 unsued for gamma1
    P_tot = s% Prad(k) + s% Pgas(k) ! s% P flag is broken for some reason, so we calculate manually
    integral_norm = integral_norm + P_tot*s% dm(k)/s% rho(k)
    gamma1_integral = gamma1_integral + &
    (s% gamma1(k)-4.d0/3.d0)*P_tot*s% dm(k)/s% rho(k)

end do


vals(81) = safe_log10(vals(81)/Lsun)
vals(82) = safe_log10(vals(82)/Lsun)
vals(83) = safe_log10(vals(83)/Lsun)
vals(84) = safe_log10(vals(84)/Lsun)
vals(85) = safe_log10(vals(85)/Lsun)
gamma1_integral = gamma1_integral/max(1d-99,integral_norm)
vals(86) = gamma1_integral







eps_beta = 0
nps_beta = 0
eps_invbeta = 0
nps_invbeta = 0

do i = 1,num_weak_reactions
    if (i <= 24) then
        nps_beta =  vals(i)
        eps_beta =  vals(i+num_weak_reactions)
    else
        nps_invbeta =  vals(i)
        eps_invbeta =  vals(i+num_weak_reactions)
    end if
 !   write (*,*) 'i = ', i
    vals(87) = vals(87) + eps_beta
    vals(88) = vals(88) + eps_invbeta
    vals(89) = vals(89) + nps_beta
    vals(90) = vals(90) + nps_invbeta
!write (*,*) 'eps_beta', eps_beta
!write (*,*) 'total so far',vals(303)

end do


vals(87) = safe_log10(vals(87)/Lsun)! units of Lsun
vals(88) = safe_log10(vals(88)/Lsun) ! units of Lsun
vals(89) = safe_log10(vals(89)) ! units of 1/s
vals(90) = safe_log10(vals(90)) ! units of 1/s

end subroutine data_for_extra_history_columns

      
      integer function how_many_extra_profile_columns(id)
         use star_def, only: star_info
         integer, intent(in) :: id
         integer :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         how_many_extra_profile_columns = 1!2
      end function how_many_extra_profile_columns
      
      
      subroutine data_for_extra_profile_columns(id, n, nz, names, vals, ierr)
         use star_def, only: star_info, maxlen_profile_column_name
         use const_def, only: dp
         integer, intent(in) :: id, n, nz
         character (len=maxlen_profile_column_name) :: names(n)
         real(dp) :: vals(nz,n)
         integer, intent(out) :: ierr
         type (star_info), pointer :: s
         integer :: k
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
        names(1) = 'zbar_div_abar'
        !names(2) = 'end_eps_neu_nuc'
        do k=1,s% nz
           vals(k,1) = s% zbar(k)/s% abar(k)
           !vals(k,2) = s% xtra1_array(k)
         !  if (vals(k,2)> 0d0):
        end do
      end subroutine data_for_extra_profile_columns

  
      include 'gyre_in_mesa_extras_finish_step.inc'
     

      ! returns either keep_going or terminate.
      integer function extras_finish_step(id)
         integer, intent(in) :: id
         integer :: ierr
         type (star_info), pointer :: s
         ierr = 0
         call star_ptr(id, s, ierr)
         if (ierr /= 0) return
         extras_finish_step = keep_going
         if (.not. s% x_logical_ctrl(37)) return
         extras_finish_step = gyre_in_mesa_extras_finish_step(id)
         if (extras_finish_step == terminate) &
             s% termination_code = t_extras_finish_step
      end function extras_finish_step
      
! std_reaction_neuQs is from reaclib or weak_info.list

    subroutine do2_net( &
    s, k, species, num_reactions, net_lwork, &
    n, nz, vals, frac, names_weak_reactions, ierr)
    use rates_def, only: std_reaction_Qs, std_reaction_neuQs, i_rate, reaction_data
    use net_def, only: Net_Info
    use net_lib, only: net_get_with_Qs
    use chem_def, only: chem_isos, category_name
    use eos_def, only : i_eta
    use utils_lib,only: &
    is_bad_num, realloc_double, realloc_double3
    type (star_info), pointer :: s
    integer, intent(in) :: k, species, num_reactions, net_lwork, n, nz
    character (len=maxlen_history_column_name) :: names_weak_reactions(n)
    real(dp) :: vals(nz,n)
    real(dp) :: frac(n)
    integer, intent(out) :: ierr

    integer :: i, j, screening_mode
    real(dp) :: log10_rho, log10_T, alfa, beta, &
    d_eps_nuc_dRho, d_eps_nuc_dT, cat_factor
    real(dp), target :: net_work_ary(net_lwork)
    real(dp), pointer :: net_work(:)
    type (Net_Info), target :: net_info_target
    type (Net_Info), pointer :: netinfo

    character (len=100) :: message
    !real(dp), pointer :: reaction_neuQs(:)
    real(dp) :: eps_neu_tot ! ergs/g/s neutrinos from weak reactions
    real(dp), pointer, dimension(:) :: actual_Qs_p, actual_neuQs_p ! ignore if null  (num_reactions)
    logical, pointer :: from_weaklib_p(:) ! ignore if null
    integer :: sz
    real(dp) :: eps_nuc_factor

    logical, parameter :: dbg = .false.

    include 'formats'


    ierr = 0

    net_work => net_work_ary
    netinfo => net_info_target


     actual_neuQs_p => actual_neuQs
     actual_Qs_p => actual_Qs
     from_weaklib_p => from_weaklib

    log10_rho = s% lnd(k)/ln10
    log10_T = s% lnT(k)/ln10

    screening_mode = get_screening_mode(s,ierr)
    if (ierr /= 0) then
    write(*,*) 'unknown string for screening_mode: ' // trim(s% screening_mode)
    stop 'do2_net'
    return
    end if

    call net_get_with_Qs( &
    s% net_handle, .false., netinfo, species, num_reactions, s% xa(1:species,k), &
    s% T(k), log10_T, s% rho(k), log10_Rho, &
    s% abar(k), s% zbar(k), s% z2bar(k), s% ye(k), &
    s% eta(k), s% d_eos_dlnT(i_eta,k) , s% d_eos_dlnd(i_eta,k), &
    s% rate_factors, s% weak_rate_factor, &
    std_reaction_Qs, std_reaction_neuQs, .false., .false., &
    s% eps_nuc(k), d_eps_nuc_dRho, d_eps_nuc_dT, s% d_epsnuc_dx(:,k), &
    s% dxdt_nuc(:,k), s% d_dxdt_nuc_dRho(:,k), s% d_dxdt_nuc_dT(:,k), s% d_dxdt_nuc_dx(:,:,k), &
    screening_mode, s% theta_e(k), &
    s% eps_nuc_categories(:,k), &
    eps_neu_tot, net_lwork,net_work,actual_Qs_p, actual_neuQs_p, from_weaklib_p, ierr)
    

    if (ierr /= 0) then
    write(*,*) 'do2_net: net_get failure for cell ', k
    return
    end if
    call show_stuff1(s,k,net_lwork,net_work,n,nz,vals,frac,names_weak_reactions)

    end subroutine do2_net




    integer function get_screening_mode(s,ierr)
    use rates_lib, only: screening_option
    type (star_info), pointer :: s
    integer, intent(out) :: ierr
    include 'formats'
    ierr = 0
    if (s% screening_mode_value >= 0) then
    get_screening_mode = s% screening_mode_value
    return
    end if
    get_screening_mode = screening_option(s% screening_mode, ierr)
    if (ierr /= 0) return
    s% screening_mode_value = get_screening_mode
    ! returns a value sets which screening option to choose.
    !write(*,2) 'get_screening_mode ' // &
    !   trim(s% screening_mode), get_screening_mode
    end function get_screening_mode




    subroutine show_stuff1(s,k,lwork,work,n,nz,vals,frac,names_weak_reactions)
    use chem_def
    use rates_def
    use rates_lib, only: rates_reaction_id
    use net_lib, only: get_reaction_id_table_ptr, get_net_rate_ptrs
    use net_def, only: Net_Info, Net_General_Info, get_net_ptr
    use math_lib, only: safe_log10
    type (star_info), pointer :: s
    integer, intent(in) :: k, lwork, n, nz!, num_weak_reactions
    character (len=maxlen_history_column_name) :: names_weak_reactions(n)
    real(dp), pointer :: work(:)
    real(dp) :: vals(n), frac(n)
    integer :: rate_id(n)
    type (Net_General_Info), pointer :: g


    integer, pointer :: reaction_id(:) ! maps net reaction number to reaction id
    integer :: i, j, ierr, species, num_reactions!, rate_id
    real(dp), pointer, dimension(:) :: &
    rate_screened, rate_screened_dT, rate_screened_dRho, &
    rate_raw, rate_raw_dT, rate_raw_dRho
    !real(dp), pointer, dimension(:), intent(in) :: actual_Qs, actual_neuQs ! ignore if null  (num_reactions)
    !logical, pointer, intent(in) :: from_weaklib(:) ! ignore if null

    include 'formats'

    ierr = 0
    num_reactions = s% num_reactions


    call get_net_ptr(s% net_handle, g, ierr)

    call get_net_rate_ptrs(s% net_handle, &
    rate_screened, rate_screened_dT, rate_screened_dRho, &
    rate_raw, rate_raw_dT, rate_raw_dRho, lwork, work, &
    ierr)
    if (ierr /= 0) then
    write(*,*) 'failed in get_net_rate_ptrs'
    stop 1
    end if

    call get_reaction_id_table_ptr(s% net_handle, reaction_id, ierr)
    if (ierr /= 0) return

!weaklib_ids
! We calculate the first 3 rates explicitly.
    !r_h1_he3_wk_he4

    rate_id(1) = rates_reaction_id(names_weak_reactions(1))
    if (rate_id(1) <= 0) then
    write(*,*) 'failed to find reaction rate id 1 -- not valid name?'
    vals(1) = 0
    return
    end if
    call get_rate(rate_id(1),vals(1),vals(1+40),k)
    frac(1) = s% xa(g% net_iso(reaction_inputs(4,rate_id(1))),k)


    !r_h1_h1_ec_h2
    rate_id(2) = rates_reaction_id(names_weak_reactions(2))
    if (rate_id(2) <= 0) then
    write(*,*) 'failed to find reaction rate id 2 -- not valid name?'
    vals(2) = 0
    return
    end if
    call get_rate(rate_id(2),vals(2),vals(2+40),k)
    frac(2) = s% xa(g% net_iso(reaction_inputs(1,rate_id(2))),k)
    !write(*,*) 'h1 ', frac(2) ! check h1 frac

    !r_h1_h1_wk_h2
    rate_id(3) = rates_reaction_id(names_weak_reactions(3))
    if (rate_id(3) <= 0) then
    write(*,*) 'failed to find reaction rate id 3 -- not valid name?'
    vals(3) = 0
    return
    end if
    call get_rate(rate_id(3),vals(3),vals(3+40),k)
    frac(3) = s% xa(g% net_iso(reaction_inputs(1,rate_id(3))),k)

    ! all other reactions
    do i = 4,40
        rate_id(i) = rates_reaction_id(names_weak_reactions(i))
        if (rate_id(i) <= 0) then
        write(*,*) 'failed to find reaction rate id ', i ,' -- not valid name?'
        vals(i) = 0
        return
        end if
        call get_rate(rate_id(i),vals(i),vals(i+40),k)
        frac(i) = s% xa(g% net_iso(reaction_inputs(2,rate_id(i))),k)
       ! write(*,*)' i = ', i
        !write(*,*) 'vals(i+148), neuqs', vals(i+148)
    end do

    !write (*,*) 'k', k
    !write(*,*) 'he3', he3_frac
    !write(*,*) 'h1', h1_frac
    !write(*,*) 'be7', be7_frac
    !write(*,*) 'ne19', ne19_frac
    !write(*,*) 'si27', si27_frac



    contains

    subroutine get_rate(id, get_rate1, get_rate_Qneu,k1)
    integer, intent(in) :: id
    !real(dp), intent(in) :: get_rate1, get_rate_Qneu
    integer :: j, k1
    real(dp), intent(out):: get_rate1, get_rate_Qneu

    include 'formats'

    get_rate1 = -99
    get_rate_Qneu = 0
    do j=1,num_reactions
    if (reaction_id(j) /= id) cycle
    !write(*,3) 'screened rate ' // trim(reaction_Name(reaction_id(j))), &
    !   j, k, rate_screened(j)
    if (rate_screened(j) < 1d-60) then
    get_rate1 = -99d0
    get_rate_Qneu = 0
    else
    get_rate1 = safe_log10(rate_screened(j)) ! or rate_raw(j)
    get_rate_Qneu = Qconv * actual_neuQs(j) !std_reaction_neuQs(id)

    if (k1 == s% nz) then
    !write (*,*) s% weak_rate_factor !std_reaction_neuQs(id)
    end if
    end if
    return
    end do

    write(*,*) 'failed to find reaction rate id -- not in current net?'
    get_rate1 = -99
    get_rate_Qneu = 0
    end subroutine get_rate
    end subroutine show_stuff1

      end module run_star_extras
      
